#include "sched.hh"

Schedule::Schedule(int _omega)
  : omega(_omega)
{
  s.push_back(std::vector< slot_t >(_omega));
  std::vector< slot_t > & tmp = s.back();
  for(int i = 0; i < omega; i++) {
    tmp[i].first = NULL;
  }
}

std::pair< bool, int > 
Schedule::insert(TargetInstruction * _target_inst,
		 std::map< char *, std::pair< char *, lcc::Node>, ltstr > & _inst_instance,
		 pattern_t *_pattern, int _offset)
{

  //debug
//   for(std::map< char *, char *, ltstr >::const_iterator iter = _inst_instance.begin();
// 	    iter != _inst_instance.end(); ++iter) {
//     char *str1 = iter->first;
//     char *str2 = iter->second;
//     std::cout << str1 << ":" << str2 << std::endl;
//   }
  //

  advance(_offset);

  for(int i = 0; i < omega; i++) {
    std::vector< slot_t > & tmp = s.back();
    if(tmp[i].first == NULL) {
      //empty slot
      tmp[i].first = _pattern;
      tmp[i].second.first = _target_inst;
      tmp[i].second.second = _inst_instance;
      return std::make_pair(true, 0);
    }
  }

  s.push_back(std::vector< slot_t >(omega));
  std::vector< slot_t > & tmp = s.back();
  for(int j = 0; j < omega; j++) {
    tmp[j].first = NULL;
  }
  tmp[0].first = _pattern;
  tmp[0].second.first = _target_inst;
  tmp[0].second.second = _inst_instance;

  return std::make_pair(true, 1);
}

Schedule::~Schedule()
{
}

Schedule::Schedule(const Schedule & _s)
{
  omega = _s.omega;
  s = _s.s;
}

std::ostream & operator<<(std::ostream & _out, const Schedule & _s)
{
  int length = _s.s.size();
  int omega = _s.s[0].size();
  std::vector< std::vector< Schedule::slot_t > > s = (_s.s);
  for(int i = 0; i < length; i++) {
    for(int j = 0; j < omega; j++) {
      Schedule::slot_t & slot = s[i][j];
      if(slot.first) {
	for(std::map< char *, std::pair< char *, lcc::Node >, ltstr >::const_iterator iter = slot.second.second.begin();
	    iter != slot.second.second.end(); ++iter) {
	  char *str1 = iter->first;
	  char *str2 = iter->second.first;
	  _out << str1 << ":" << str2 << std::endl;
	}
      }
    }
  }
  return _out;
  /*
  for(int j = 0; j < _s.s.size(); j++) {
    for(int i = 0; i < _s.s[j].size(); i++) {
      if(_s.s[j][i].first) {
	_out << "EMPTY";
      } else {
	_out << *(_s.s[j][i].second.first);
      }
      _out << " || ";
    }
    _out << std::endl;
  }
  return _out;
  */
}

void Schedule::advance(int _offset)
{
  if(_offset > 0) {
    for(int i = 0; i < _offset; i++) {
      s.push_back(std::vector< slot_t >(omega));
      std::vector< slot_t > & tmp = s.back();
      for(int j = 0; j < omega; j++) {
	tmp[j].first = NULL;
      }
    }
  }
}

bool Schedule::available_slot(void)
{
  std::vector< slot_t > & tmp = s.back();
  for(int i = 0; i < omega; i++) {
    if(tmp[i].first == NULL) {
      return true;
    }
  }
  return false;
}
